package imageFiltering.filter;

import imageFiltering.*;
import imageFiltering.filter.*;
import java.util.Random;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;

public class EdgeFilterSP extends Filter {
	
    private int SIGMA = 1000,
	SIGMAMax = 2000,
	SIGMAMin = 0;
	
    protected static final int[][] mask = {{-1,-1,-1},
					   {-1, 8,-1},
					   {-1,-1,-1}};

    float near_init = 30.0f;
    boolean white_flag = false;
    float border = 1000;

    public String getName()
    {
	return "EdgeFilterSP";
    }
	
    public void filter(AnimatedImage AI)
    {
	Random rand = new Random();
	int width = AI.getWidth();
	int height = AI.getHeight();
	int[] edges = new int[width*height];

	AI.resetImage();
	
	for(int y = 0; y < height; y++)
	    for(int x = 0; x < width; x++)
		AI.setRGB(x, y, AI.getR(x, y), AI.getG(x, y), AI.getB(x, y));
	
	AI.repaintAll();
	for (int y = 0; y < height; y++) {
	    for(int x = 0; x < width; x++){
		int l=0;
		int R = AI.getR(x, y);
		int G = AI.getG(x, y);
		int B = AI.getB(x, y);
		int L = getLuminance(R,G,B);
		int Cr = getCr(R,G,B);
		int Cb = getCb(R,G,B);
		final int offset = mask.length/2;
		for(int i = 0; i < mask.length; i++){
		    if( y+i-offset < 0 || y+i-offset >= height)
			break;
		    for(int j = 0; j < mask[i].length; j++){
			if( x+j-offset < 0 || x+j-offset >= width)
			    break;
			int lR = AI.getR(x + j - offset, y + i - offset);
			int lG = AI.getG(x + j - offset, y + i - offset);
			int lB = AI.getB(x + j - offset, y + i - offset);
			int lL = getLuminance(lR,lG,lB) - L;
			int lCr = getCr(lR,lG,lB) - Cr;
			int lCb = getCb(lR,lG,lB) - Cb;
			l += (lL*lL + lCr*lCr + lCb*lCb);
		    }
		}
		edges[x+y*width] = l;
	    }
	}
	for (int y = 0; y < height; y++) {
	    for(int x = 0; x < width; x++){
		if(edges[x+y*width]/mask[mask.length/2][mask[mask.length/2].length/2] >= border){
		    AI.setRGB(x,y,0,0,0);
		}else{
		    if(white_flag){
			AI.setRGB(x,y,255,255,255);
		    }
		}
	    }
	    this.switchThread();
	    }
	this.switchThread();
	AI.repaintAll();
	return;	
    }
    
    public int getLuminance(int r, int g, int b)
    {
	int ret = (int)(0.3f*(float)r + 0.59f*(float)g + 0.11f*(float)b);
	if(ret < 0) ret = 0;
	if(ret > 255) ret = 255;
	return ret;
    }
    
    public int getCr(int r,int g,int b){
	int ret = (int)(0.5f*(float)r - 0.42f*(float)g - 0.08f*(float)b + 128.0f);
	if(ret < 0) ret = 0;
	if(ret > 255) ret = 255;
	return ret;
    }

    public int getCb(int r,int g,int b){
	int ret = (int)(-0.17f*(float)r - 0.33f*(float)g + 0.5f*(float)b + 128.0f);
	if(ret < 0) ret = 0;
	if(ret > 255) ret = 255;
	return ret;
    }

    class EdgeFilterSPPanel extends FilterPanel implements AdjustmentListener{

	Label L1 = new Label("Edge Filter parameter"),
	    L2 = new Label("Edge parameter");

	    TextField T1 = new TextField(""+SIGMA,5);
	Scrollbar S1 = new Scrollbar(Scrollbar.HORIZONTAL,1000,20,20,2000);

	EdgeFilterSPPanel(){

	this.setLayout(new GridLayout(20,1));
	this.add(new Container());
	this.add(L1);
	this.add(L2);
	this.add(S1);
	this.add(T1);
	S1.addAdjustmentListener(this);
	}
	
	public void setFilterParameter(){
		
	    try{
		border = Integer.parseInt(T1.getText());
	    }catch(NumberFormatException e){}
	    if(SIGMA > SIGMAMax) SIGMA = SIGMAMax;
	    if(SIGMA < SIGMAMin) SIGMA = SIGMAMin;
	    T1.setText(""+SIGMA);
	}
	
	public void adjustmentValueChanged(AdjustmentEvent e)
	{
	    int val;
	    Scrollbar se = (Scrollbar) e.getSource();
	    if(se == S1){
		val = S1.getValue();

	       T1.setText(""+val);
	    }
	}  
    }
	
    public FilterPanel getFilterPanel(){
	return new  EdgeFilterSPPanel();
    }
}
